from AccessControl import getSecurityManager
from five import grok
from zope import schema
from zope.globalrequest import getRequest
from zope.schema.interfaces import IField
from zope.schema.interfaces import IPassword
from plone.directives import form
from plonetheme.nuplone import MessageFactory as _
from z3c.form.interfaces import IDataManager
from z3c.form.interfaces import NO_VALUE
from Products.PluggableAuthService.interfaces.authservice import IPropertiedUser
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage

class ISettings(form.Schema):
    fullname = schema.TextLine(
            title=_("label_fullname", default=u"Full name"),
            required=True)

    email = schema.ASCIILine(
            title = _("label_email", default="Email address"),
            required=True)

    password = schema.Password(
            title = _("label_password", default=u"Password"),
            required = True)


class UserPropertyDataManager(grok.MultiAdapter):
    grok.adapts(IPropertiedUser, IField)
    grok.implements(IDataManager)

    def __init__(self, user, field):
        self.user=user
        self.field=field
        self.propname=field.__name__
        for sheet_id in user.listPropertysheets():
            sheet=user.getPropertysheet(sheet_id)
            if sheet.hasProperty(self.propname):
                self.propertysheet=sheet
                break
        else:
            self.propname=None


    def get(self):
        return self.propertysheet.getProperty(self.propname)

    def query(self, default=NO_VALUE):
        if self.propname is None:
            return default
        return self.propertysheet.getProperty(self.propname, NO_VALUE)

    def set(self, value):
        self.propertysheet.setProperty(self.user, self.propname, value)

    def canAccess(self):
        return self.propname is not None

    def canWrite(self):
        return self.propname is not None



class UserPasswordDataManager(grok.MultiAdapter):
    grok.adapts(IPropertiedUser, IPassword)
    grok.implements(IDataManager)

    def __init__(self, user, field):
        self.user=user
        self.field=field

    def get(self):
        raise AttributeError(self.field.__name__)

    def query(self, default=NO_VALUE):
        return default

    def set(self, value):
        if value is None:
            return IStatusMessage(getRequest()).add(
                _('Password not updated, none was specified.'),
                type='error')

        try:
            mt = getToolByName(self.user, "portal_membership")
        except AttributeError:
            return IStatusMessage(getRequest()).add(
                _('Cannot change password for Zope users, only Plone'),
                type='error')
        else:
            mt.setPassword(value)

    def canAccess(self):
        return False

    def canWrite(self):
        return True


class Settings(form.SchemaEditForm):
    grok.context(ISiteRoot)
    grok.name("settings")

    schema = ISettings

    def getContent(self):
        return getSecurityManager().getUser()





from zope.interface import implements
from App.class_init import default__class_init__ as InitializeClass
from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.interfaces.plugins import IChallengePlugin
from plonetheme.nuplone.skin.interfaces import NuPloneSkin


class LoginChallenger(BasePlugin):
    """Simple login challenger which does nothing, allowing the
    Unauthorized exception view to be used instead.
    """
    meta_type = "NuPlone Login Challenger"
    id = "nuplone-challenger"
    implements(IChallengePlugin)

    def challenge(self, request, response):
        if not NuPloneSkin.providedBy(request):
            return False

        response.setStatus(403)
        response._locked_status = True
        return True


InitializeClass(LoginChallenger)




_applied = False

if not _applied:
    from z3c.form.form import Form
    # Zap the totally useless 'There were some errors' message.
    Form.formErrorsMessage = None
    del Form
    _applied=True




from zope.interface import Interface
from five import grok
from plonetheme.nuplone.skin.interfaces import NuPloneSkin

grok.templatedir("templates")

class Layout(grok.View):
    grok.context(Interface)
    grok.name("layout")
    grok.layer(NuPloneSkin)
    grok.template("layout")





import unittest2 as unittest
from plonetheme.nuplone.testing import NUPLONE_FUNCTIONAL_TESTING
from plone.testing.z2 import Browser


class PasswordResetTests(unittest.TestCase):
    layer = NUPLONE_FUNCTIONAL_TESTING

    # XXX Disabled for now because grok-style views are not working in
    # the plone.app.testing-based setup.
    def XtestNoKey(self):
        browser = Browser(self.layer["app"])
        browser.open("%s/reset-password" % self.layer["portal"].absolute_url())
        browser.getForm().submit()

    def testInvalidKey(self):
        from zExceptions import NotFound
        browser = Browser(self.layer["app"])
        browser.handleErrors = False
        url = "%s/reset-password/bogus" % self.layer["portal"].absolute_url()
        self.assertRaises(NotFound, browser.open, url)

    def testDoubleKeys(self):
        from zExceptions import NotFound
        browser = Browser(self.layer["app"])
        browser.handleErrors = False
        url = "%s/reset-password/one/two" % self.layer["portal"].absolute_url()
        self.assertRaises(NotFound, browser.open, url)



import logging
from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_chain
from Acquisition import aq_parent
from five import grok
from zope.browser.interfaces import IBrowserView
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
import zExceptions


log=logging.getLogger(__name__)

grok.templatedir("templates")


class Error(grok.View):
    grok.context(Exception)
    grok.layer(NuPloneSkin)
    grok.name("index.html")
    grok.template("error_generic")

    def update(self):
        self.exception=aq_inner(self.context)
        self.context=context=aq_parent(self)
        if IBrowserView.providedBy(context):
            # NotFound errors can have extra aq wrapping
            self.context=context=aq_parent(context)
        try:
            log.exception("Error at %s", repr(context))
        except zExceptions.Unauthorized:
            pass



class NotFound(Error):
    grok.context(zExceptions.NotFound)
    grok.template("error_notfound")



class Unauthorized(Error):
    grok.context(zExceptions.Unauthorized)
    grok.template("error_unauthorized")

    def authenticate(self):
        """Try to authenticate the user manually, since ZPublisher dropped the
        user whenn it failed to validate access."""
        for parent in aq_chain(aq_inner(self.context)):
            if hasattr(aq_base(parent), "acl_users"):
                uf=parent.acl_users
                try:
                    return uf.validate(self.request, None, roles=["Anonymous"])
                except zExceptions.Unauthorized:
                    pass


    def update(self):
        super(Unauthorized, self).update()
        self.authenticate()




from plone.app.testing import PloneWithPackageLayer
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting
import plonetheme.nuplone

FIXTURE = PloneWithPackageLayer(
        zcml_filename="configure.zcml",
        zcml_package=plonetheme.nuplone,
        gs_profile_id="plonetheme.nuplone:default",
        name="plonetheme.nuplone:fixture")

NUPLONE_INTEGRATION_TESTING = IntegrationTesting(
        bases=(FIXTURE,), name="NuPlone:Integration")
NUPLONE_FUNCTIONAL_TESTING = FunctionalTesting(
        bases=(FIXTURE,), name="NuPlone:Functional")


__all__ = ['NUPLONE_INTEGRATION_TESTING', 'NUPLONE_FUNCTIONAL_TESTING']



from Acquisition import aq_inner
from five import grok
from plone.tiles import Tile
from plone.folder.interfaces import IExplicitOrdering
from plone.folder.interfaces import IOrderableFolder
from plonetheme.nuplone.skin.interfaces import NuPloneSkin


class UpdateOrder(grok.View):
    grok.context(IOrderableFolder)
    grok.name("update-order")
    grok.require("cmf.ModifyPortalContent")
    grok.layer(NuPloneSkin)

    def render(self):
        order=self.request.form.get("order[]")
        if not order or not isinstance(order, list):
            return

        orderer=IExplicitOrdering(aq_inner(self.context))
        for (pos, id) in enumerate(order):
            id=id.split("-", 1)[1]
            orderer.moveObjectToPosition(id, pos)

        return


class OrderingTile(Tile):
    def __call__(self):
        if not IOrderableFolder.providedBy(self.context):
            return None
        return self.index()




# -*- coding: UTF-8 -*-
""" StatusMessage adapter tests.
"""
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.interface import directlyProvides
from zope.publisher.base import RequestDataProperty, RequestDataMapper
from zope.publisher.browser import TestRequest as TestRequestBase

from webhelpers.html.builder import literal

import Products.Five
from Products.Five import fiveconfigure
from Products.Five import zcml
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase import layer 
from Products.statusmessages.interfaces import IStatusMessage

from plonetheme.nuplone.skin.interfaces import NuPloneSkin

SiteLayer = layer.PloneSite

class CookieMapper(RequestDataMapper):
    _mapname = '_cookies'

    def __setitem__(self, key, value):
        pass

class TestRequest(TestRequestBase):
    """Zope 3's TestRequest doesn't support item assignment, but Zope 2's
    request does.
    """
    def __setitem__(self, key, value):
        pass
        
    cookies = RequestDataProperty(CookieMapper)


class BaseLayer(SiteLayer):

    @classmethod
    def setUp(cls):
        """ Set up the additional products required
        """
        PRODUCTS = [
                'plonetheme.nuplone',
                ]
        ptc.setupPloneSite(products=PRODUCTS)

        fiveconfigure.debug_mode = True
        import plonetheme.nuplone
        zcml.load_config('meta.zcml', Products.Five)
        zcml.load_config('configure.zcml', Products.statusmessages)
        zcml.load_config('configure.zcml', plonetheme.nuplone)
        fiveconfigure.debug_mode = False
        SiteLayer.setUp()


class TestHTMLStatusMessages(ptc.PloneTestCase):
    layer = BaseLayer 

    def testAdapter(self):
        """ Test status messages
            First some boilerplate.
        """
        request = self.request = TestRequest()

        directlyProvides(request, NuPloneSkin)

        # Now lets make sure we can actually adapt the request.
        status = IStatusMessage(self.request)
        self.assertEquals(IStatusMessage.providedBy(status), True)
        assert(hasattr(status, 'addHTMLStatusMessage'))

        # We also need the request to be annotatable:
        directlyProvides(self.request, IAttributeAnnotatable)

        # Make sure there's no stored message.
        self.assertEquals(len(status.show()), 0)

        # Add one message
        status.add(u'test', type=u'info')

        # Now check the results
        messages = status.show()
        self.assertEquals(len(messages), 1)
        self.assertEquals(messages[0].message, u'test')
        self.assertEquals(messages[0].type, u'info')

        # Make sure messages are removed
        self.assertEquals(len(status.show()), 0)

        # Since we accessed the message prior to publishing the page, we must 
        # ensure that the messages have been removed from the cookies
        self.assertEquals(len(status.show()), 0)

        # Now we repeat the test, only this time we publish the page prior to
        # retrieving the messages

        # Add one message
        status.add(u'test', type=u'info')

        # Now check the results
        messages = status.show()
        self.assertEqual(len(messages), 1)
        self.assertEquals(messages[0].message, u'test')
        self.assertEquals(messages[0].type, u'info')

        # Make sure messages are removed
        self.assertEquals(len(status.show()), 0)

        # Add two messages
        status.add(u'test', type=u'info')
        status.add(u'test1', u'warn')
        
        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 2)
        test = messages[1]
        self.assertEquals(test.message, u'test1')
        self.assertEquals(test.type, u'warn')

        # Make sure messages are removed again
        self.assertEquals(len(status.show()), 0)

        # Add two identical messages
        status.add(u'test', type=u'info')
        status.add(u'test', type=u'info')

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.message, u'test')
        self.assertEquals(test.type, u'info')

        # Make sure messages are removed again
        self.assertEquals(len(status.show()), 0)

        # Test incredibly long messages:
        status.add(u'm' * 0x400, type=u't' * 0x20)

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 1)

        test = messages[0]
        assert(test.message == u'm' * 0x3FF)
        assert(test.type == u't' * 0x1F)

        # Add one HTML messages
        status.addHTML(u'test', type=u'success')
        
        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.type, u'success')
        self.assertEquals(test.message, u'test')
        self.assertEquals(type(test.message), literal)

        # Add two HTML messages
        status.addHTML(u'test', type=u'info')
        status.addHTML(u'test1', u'warn')
        
        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 2)
        test = messages[1]
        self.assertEquals(test.message, u'test1')
        self.assertEquals(type(test.message), literal)
        self.assertEquals(test.type, u'warn')

        # add two identical messages
        status.addHTML(u'test', type=u'info')
        status.addHTML(u'test', type=u'info')

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.message, u'test')
        self.assertEquals(type(test.message), literal)
        self.assertEquals(test.type, u'info')

        # Make sure messages are removed again
        self.assertEquals(len(status.show()), 0)

        # Test incredibly long messages:
        status.addHTML(u'm' * 0x400, type=u't' * 0x20)

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 1)

        test = messages[0]
        self.assertEquals(type(test.message), literal)
        assert(test.message == '%s' % (u'm' * 0x3FF))
        assert(test.type == u't' * 0x1F)


        # Add two mixed messages
        status.add(u'test', type=u'info')
        status.addHTML(u'test1', u'warn')
        
        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 2)
        test = messages[0]
        self.assertEquals(test.message, u'test')
        self.assertNotEquals(type(test.message), literal)
        self.assertEquals(test.type, u'info')

        test = messages[1]
        self.assertEquals(test.message, u'test1')
        self.assertEquals(type(test.message), literal)
        self.assertEquals(test.type, u'warn')

        # Add a more complicated html message
        status.addHTML(u'You can go <a href="http://plone.org">here</a>.', type=u'success')
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.type, u'success')
        self.assertEquals(
            test.message, 
            u'You can go <a href="http://plone.org" rel="nofollow" target="_blank">here</a>.')
        self.assertEquals(type(test.message), literal)

        # Add html message with disallowed tags
        status.addHTML(u'<p>You can go <a href="http://plone.org">here</a>.</p>', type=u'success')
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.type, u'success')
        self.assertEquals(
            test.message, 
            u'You can go <a href="http://plone.org" rel="nofollow" target="_blank">here</a>.')
        self.assertEquals(type(test.message), literal)

        status.addHTML(u"<script type=\"javascript\">alert('hello')</script>", type=u'success')
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.type, u'success')
        self.assertEquals(test.message, '')
        self.assertEquals(type(test.message), literal)

        status.addHTML(u'<a href="data:text/html;base64,PHNjcmlwdD5hbGVydCgidGVzdCIpOzwvc2NyaXB0Pg==">click me</a>', type=u'success')
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.type, u'success')
        self.assertEquals( test.message, '<a href="">click me</a>')
        self.assertEquals(type(test.message), literal)



import logging
from chameleon.tales import StringExpr
from chameleon.astutil import Symbol
from chameleon.astutil import Static
from chameleon.codegen import template
from zope.contentprovider.interfaces import ContentProviderLookupError
from plonetheme.nuplone.tiles.tile import getTile
from plonetheme.nuplone.utils import SimpleLiteral


log = logging.getLogger(__name__)


class TileProviderTraverser(object):
    def __call__(self, context, request, name):
        tile = getTile(context, request, name)
        if tile is None:  # XXX Use custom exception?
            log.warn('Request for unknown tile %s', name)
            return u''
        return SimpleLiteral(tile())


class TileExpression(StringExpr):
    render_tile = Static(
            template("cls()", cls=Symbol(TileProviderTraverser), mode='eval'))

    def __call__(self, target, engine):
        assignment = super(TileExpression, self).__call__(target, engine)
        return assignment + template(
                'target = render_tile(context, request, target.strip())',
                target=target,
                render_tile=self.render_tile)


import five.pt.engine
five.pt.engine.Program.secure_expression_types['tile'] = TileExpression
five.pt.engine.Program.expression_types['tile'] = TileExpression



import re
from chameleon.codegen import template
from chameleon.astutil import Symbol
from chameleon.tales import StringExpr
from plonetheme.nuplone.utils import checkPermission


class PermissionExpr(StringExpr):
    def __call__(self, target, engine):
        assignment = super(PermissionExpr, self).__call__(target, engine)
        return assignment + template(
                'target = check_permission(context, target)',
                target=target, check_permission=Symbol(checkPermission))


import five.pt.engine
five.pt.engine.Program.secure_expression_types['permission'] = PermissionExpr
five.pt.engine.Program.expression_types['permission'] = PermissionExpr



import collections
from zope.component import queryMultiAdapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import adapts
from zope.interface import Interface
from zope.interface import implements
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.tiles import Tile
from plonetheme.nuplone.utils import getNavigationRoot
from plone.i18n.normalizer.interfaces import IIDNormalizer
from Products.CMFPlone.utils import typesToList
from Products.CMFCore.utils import getToolByName


class INavtreeFactory(Interface):
    """Marker interface for catalog builder functions."""

    def __call__(context, request):
        """Return a CatalogNavTree instance."""


class CatalogNavTree(object):
    def __init__(self, context, request):
        self.build(context, request)

    def build(self, context, request):
        context=aq_inner(context)

        # If we are at a default page use the folder as context for the navtree
        container=aq_parent(context)
        isp=queryMultiAdapter((container, request), name="default_page", default=None)
        if isp is not None and isp.isDefaultPage(context):
            context=container

        contextPath="/".join(context.getPhysicalPath())
        contextPathLen=len(contextPath)
        parentDepth=(contextPath.count("/")-1)
        navrootPath="/".join(getNavigationRoot(context).getPhysicalPath())

        query={}
        query["path"]=dict(query=contextPath,
                           navtree=True,
                           navtree_start=navrootPath.count("/"))
        query["portal_type"]=typesToList(context)
        query["sort_on"]="getObjPositionInParent"
        query["sort_order"]="asc"

        catalog=getToolByName(context, "portal_catalog")
        results=catalog.searchResults(query)
        cache={}
        cache[navrootPath]={"current": False, "currentParent": True, "children": []}
        for brain in results:
            path=brain.getPath()
            pathLen=len(path)
            parentPath=path.rsplit("/", 1)[0]
            ancestor=current=currentParent=False
            if path==contextPath:
                current=True
            elif contextPathLen>pathLen:
                ancestor=contextPath.startswith(path+"/")
                currentParent=ancestor and path.count("/")==parentDepth

            if brain.exclude_from_nav and not currentParent:
                continue

            oldNode=cache.get(path, None)
            node={"brain": brain,
                  "path" : path,
                  "current" : current,
                  "currentParent" : currentParent,
                  "ancestor": ancestor }

            oldNode=cache.get(path, None)
            if oldNode is not None:
                oldNode.update(node)
                node=oldNode
            else:
                node["children"]=[]
                cache[path]=node

            parentNode=cache.get(parentPath, None)
            if parentNode is None:
                parentNode=cache[parentPath]=dict(children=[node])
            else:
                parentNode["children"].append(node)
            node["parent"]=parentNode


        self.tree=cache
        self.root=cache[navrootPath]


    def __iter__(self):
        """Breadth-first iterator for navtree nodes which allows
        modifications of the tree during iteration. Modifications
        are given by passing a command to the next() method of the
        generator. For example:

        >>> tree=CatalogNavTree(context, request)
        >>> g=tree.iter()
        >>> value=g.next()
        >>> try:
        ...     while True:
        ...        if value.portal_type=="Collection":
        ...            value=g.send("prune")
        ...        else:
        ...            value=g.next()
        ... except StopIteration:
        ...     pass

        The supported commands are:

        * purge: remove the node and all its children from the tree
        * prune: remove all children of this node from the tree

        In addition you may also modify the datastructures directly
        during iteration.
        """
        queue=collections.deque([self.root])
        while queue:
            node=queue.popleft()
            action=(yield node)
            if action=="purge":
                node["parent"]["children"].remove(node)
                continue
            elif action=="prune":
                node["children"]=[]
                continue

            for child in node.get("children", []):
                queue.append(child)

    iter = __iter__



class TreeFactory(object):
    implements(INavtreeFactory)
    adapts(Interface, Interface)

    def __init__(self, context, request):
        self.context=context
        self.request=request

    def __call__(self):
        return CatalogNavTree(self.context, self.request)


class NavigationTile(Tile):

    def update(self):
        portal_types=getToolByName(self.context, "portal_types")
        type_titles=dict([(fti.getId(), fti.Title()) for fti in portal_types.listTypeInfo()])

        portal_properties=getToolByName(self.context, "portal_properties")
        use_view_types=portal_properties.site_properties.typesUseViewActionInListings
        normalize=getUtility(IIDNormalizer).normalize
        treefactory=getMultiAdapter((self.context, self.request), INavtreeFactory)
        tree=treefactory()

        for node in tree.iter():
            brain=node.get("brain", None)
            if brain is None:
                continue
            node["title"]=brain.Title
            node["description"]=brain.Description or None
            node["portal_type"]=normalize(brain.portal_type)
            node["portal_type_title"]=type_titles.get(brain.portal_type, brain.portal_type)
            node["url"]="%s/view" % brain.getURL() if brain.portal_type in use_view_types else brain.getURL()
            node["review_state"]=normalize(brain.review_state)
            node["folderish"]=brain.is_folderish
            node["class"]=" ".join(filter(None,
                                    ["active" if node["current"] or node["currentParent"] else None,
                                     "current" if node["current"] else None])) or None

        if "brain" in tree.root:
            self.tree=[tree.root]
        else:
            self.tree=tree.root["children"]


    def __call__(self):
        self.update()
        return self.index()




from Acquisition import aq_inner
from AccessControl import getSecurityManager
from zope.interface import Interface
from five import grok
import Globals
from zExceptions import Unauthorized

class WhoAmI(grok.View):
    """Simple view to help debug authorisation problems. This view returns 
    the id and login name of the current user and all his roles, both global
    and within the current context.

    This view is only accessible when Zope is running in debug mode.
    """
    grok.context(Interface)
    grok.name("whoami")
    grok.require("zope2.Public")

    def update(self):
        if not Globals.DevelopmentMode:
            raise Unauthorized

    def render(self):
        user=getSecurityManager().getUser()
        output=[]
        output.append("User id: %s" % user.getId())
        output.append("Login  : %s" % user.getUserName())
        output.append("Roles  : %s" % " ".join(user.getRoles()))
        output.append("Local R: %s" % " ".join(user.getRolesInContext(aq_inner(self.context))))

        self.request.response.setHeader("Content-Type", "text/plain")
        return "\n".join(output)



from zope.dottedname.resolve import resolve

# I am not going to document this. This is a essentially workaround
# http://code.google.com/p/dexterity/issues/detail?id=123

def FieldWidgetFactory(factory, **kw):
    if isinstance(factory, basestring):
        factory=resolve(factory)
    def wrapper(field, request):
        widget=factory(field, request)
        for (key,value) in kw.items():
            setattr(widget, key, value)
        return widget
    return wrapper






from plone.tiles import Tile
from Products.statusmessages.interfaces import IStatusMessage

class StatusmessagesTile(Tile):
    mapping = dict(info="notice",
                   warn="warning")

    def update(self):
        messages=IStatusMessage(self.request).show()
        for message in messages:
            message.type=self.mapping.get(message.type, message.type)
        self.messages=messages

    def __call__(self):
        self.update()
        return self.index()



from AccessControl import getSecurityManager
from plonetheme.nuplone.tiles.tile import AppConfigTile
from plonetheme.nuplone.utils import SimpleLiteral
from plonetheme.nuplone.utils import isAnonymous

_GA_COOKIE = '_nuplone_ga'


def _ga_queue(request):
    cookie = request.cookies.get(_GA_COOKIE, '')
    return filter(None, cookie.split(','))


def trigger_extra_pageview(request, url):
    """Trigger an extra Google Analytics pageview on the next rendered page.
    """
    queue = _ga_queue(request)
    if url not in queue:
        queue.append(url)
    request.response.setCookie(_GA_COOKIE, ','.join(queue), path='/')


class AnalyticsTile(AppConfigTile):
    def _pop_ga_queue(self):
        queue = _ga_queue(self.request)
        self.request.response.expireCookie(_GA_COOKIE, path='/')
        return queue

    def __call__(self):
        self.account = self.data.get("account", None)
        if not self.account:
            return ''
        self.domain = self.data.get("domain", None)
        user = getSecurityManager().getUser()
        self.auth_status = 'anonymous' if isAnonymous(user) else 'authenticated'
        self.ga_queue = self._pop_ga_queue()
        return SimpleLiteral(self.index())



import collections
import martian
from five import grok
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserView
from zope import component
from z3c.form.interfaces import IGroup
from z3c.form.interfaces import IWidget
from z3c.form.browser.checkbox import CheckBoxWidget
from plone.autoform.form import AutoExtensibleForm
from plone.directives.form.schema import FormMetadataListStorage
from plone.directives.form.schema import Schema
from plone.directives.form.schema import TEMP_KEY
from plone.supermodel.interfaces import FIELDSETS_KEY
from plone.supermodel.utils import mergedTaggedValueList
from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity.interfaces import IDexterityContent
from plone.behavior.interfaces import IBehavior
import plone.z3cform.fieldsets.interfaces

Dependency = collections.namedtuple("Dependency", "name field op value action")
DEPENDENCY_KEY = "plonetheme.nuplone.z3cform.dependency"
LAYOUT_KEY = "plonetheme.nuplone.z3cform.layout"


class depends(martian.Directive):
    """Directive used to declare a dependency on other field values."""
    scope = martian.CLASS
    store = FormMetadataListStorage()
    key = DEPENDENCY_KEY

    def factory(self, field, name, op="on", value=None, action="show"):
        if op not in [ "on", "off", "==", "!=" ]:
            raise ValueError("Invalid operand given")
        if action not in [ "show", "enable" ]:
            raise ValueError("Invalid action given")
        return [Dependency(field, name, op, value, action)]


class FormSchemaGrokker(martian.InstanceGrokker):
    """Grok form schema hints."""
    martian.component(Schema.__class__)
    martian.directive(depends)

    def execute(self, interface, config, **kw):
        if not interface.extends(Schema):
            return False

        # Copy from temporary to real value
        directiveSupplied = interface.queryTaggedValue(TEMP_KEY, None)
        if directiveSupplied is not None:
            for key, tgv in directiveSupplied.items():
                existingValue = interface.queryTaggedValue(key, None)

                if existingValue is not None:
                    if type(existingValue) != type(tgv):
                        # Don't overwrite if we have a different type
                        continue
                    elif isinstance(existingValue, list):
                        existingValue.extend(tgv)
                        tgv = existingValue
                    elif isinstance(existingValue, dict):
                        existingValue.update(tgv)
                        tgv = existingValue

                interface.setTaggedValue(key, tgv)

            interface.setTaggedValue(TEMP_KEY, None)
        return True


class FormDependencyExtender(grok.MultiAdapter):
    grok.adapts(Interface, Interface, AutoExtensibleForm)
    grok.implements(plone.z3cform.fieldsets.interfaces.IFormExtender)
    grok.name("plonetheme.nuplone.dependency")
    order = 0

    def __init__(self, context, request, form):
        self.context=context
        self.request=request
        self.form=form

    def update(self):
        schemas = [self.form.schema]

        if IDexterityContent.providedBy(self.context):
            fti = component.getUtility(
                IDexterityFTI,
                name=self.context.portal_type)
            for name in fti.behaviors:
                behavior = component.queryUtility(IBehavior, name=name)
                if behavior and behavior.interface.extends(Schema):
                    schemas.append(behavior.interface)

        directives = []
        for schema in schemas:
            directives += mergedTaggedValueList(schema, DEPENDENCY_KEY)

        dependencies = {}
        for directive in directives:
            dependencies.setdefault(directive.name, []).append(directive)

        todo=collections.deque([self.form])
        while todo:
            group=todo.pop()
            if hasattr(group, "groups"):
                todo.extendleft(group.groups)
            for (name, field) in group.fields.items():
                depends=dependencies.get(name, None)
                if depends is None:
                    continue
                field.field._dependencies=depends


class WidgetDependencyView(grok.MultiAdapter):
    grok.adapts(IWidget, Interface)
    grok.implements(IBrowserView)
    grok.name("dependencies")

    def __init__(self, widget, request):
        self.widget=widget
        self.request=request

    def __call__(self):
        dependencies=getattr(self.widget.field, "_dependencies", None)
        if not dependencies:
            return None

        classes=[]
        widgets=self.widget.__parent__
        for dependency in dependencies:
            widget=widgets[dependency.field]
            name=widget.name
            if isinstance(widget, CheckBoxWidget):
                name="%s:list" % name
            if dependency.op in [ "on", "off"]:
                classes.append("dependsOn-%s-%s" % (name, dependency.op))
            elif dependency.op=="==":
                classes.append("dependsOn-%s-equals-%s" % (name, dependency.value))
            elif dependency.op=="!=":
                classes.append("dependsOn-%s-notEquals-%s" % (name, dependency.value))

            classes.append("dependsAction-%s" % dependency.action)

        return " ".join(classes) if classes else None


class FormLayoutExtender(grok.MultiAdapter):
    grok.adapts(Interface, Interface, AutoExtensibleForm)
    grok.implements(plone.z3cform.fieldsets.interfaces.IFormExtender)
    grok.name("plonetheme.nuplone.layout")

    order = 10

    def __init__(self, context, request, form):
        self.context=context
        self.request=request
        self.form=form

    def update(self):
        fieldsets=mergedTaggedValueList(self.form.schema, FIELDSETS_KEY)
        if not isinstance(self.form.groups, list):
            self.form.groups=list(self.form.groups)
        groups=dict([(group.__name__, (index, group)) for (index,group) in enumerate(self.form.groups)])
        for fieldset in fieldsets:
            layout=getattr(fieldset, "layout", None)
            if layout is None:
                continue
            if fieldset.__name__ not in groups:
                continue
            (index, group)=groups[fieldset.__name__]

            if not IGroup.providedBy(group):
                group=self.form.groups[index]=group(self.context, self.request, self.form)
            group.layout=layout





import re
from Acquisition import aq_inner
from plonetheme.nuplone.tiles.tile import AppConfigTile
from plonetheme.nuplone.tiles.tile import getTile
from plonetheme.nuplone.utils import viewType

_compiles = {}

class GroupTile(AppConfigTile):
    TILE_EXPR = re.compile("(?P<name>\S+)\s*(?:\[(?P<condition>.*)\])?")
    tiles = []

    def update(self):
        self.tiles=[]
        self.tag=self.data.get("wrapper", "tal:block")
        tiles=self.data.get("tiles", "").strip().splitlines()
        context=aq_inner(self.context)
        request=self.request
        globals=None
        for tile in tiles:
            result=self.TILE_EXPR.match(tile)
            if not result:
                continue
            condition=result.group("condition")
            if condition:
                compiled=_compiles.get(condition)
                if compiled is None:
                    compiled=_compiles[condition]=compile(condition, "<string>", "eval")
                if globals is None:
                    globals=dict(context=context, request=request, view_type=viewType(context, request))
                if not eval(compiled, globals):
                    continue
            tile=getTile(context, request, result.group("name"))
            self.tiles.append(tile)


    def __call__(self):
        self.update()
        if not self.tiles:
            return u""
        result=self.index()
        if not result.strip():
            return u""
        config=self.data
        wrapper=config.get("wrapper")
        if wrapper:
            prefix=[u"<%s" % wrapper]
            if config.get("id"):
                prefix.append(u' id="%s"' % config["id"])
            if config.get("class"):
                prefix.append(u' class="%s"' % config["class"])
            prefix.append(u">")
            prefix=u"".join(prefix)
            postfix=u"</%s>" % wrapper
            result=u"%s%s%s" % (prefix, result, postfix)
        return result




import unittest2 as unittest
import zope.component
from zope.interface import alsoProvides
from plone.testing import z2
from plone.app.testing import login
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from plonetheme.nuplone.testing import NUPLONE_FUNCTIONAL_TESTING

class SiteMenuTests(unittest.TestCase):
    layer = NUPLONE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.portal.REQUEST
        alsoProvides(self.request, NuPloneSkin)
        z2.login(self.layer['app']['acl_users'], SITE_OWNER_NAME)

        # Create two folders
        self.portal.invokeFactory("Folder", "source_folder")
        self.source_folder = self.portal['source_folder']

        self.portal.invokeFactory("Folder", "dest_folder")
        self.dest_folder = self.portal['dest_folder']

        # Create an object to be copied
        self.source_folder.invokeFactory("Document", "doc")

    def testActions(self):
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Contributor',))
        # Copy the object
        cp = self.source_folder.manage_copyObjects(ids=['doc'])
        self.request['__cp'] = cp

        # We cannot paste the object in the site root
        v = zope.component.getMultiAdapter(
                                (self.portal, self.request), name="sitemenu")
        self.assertIsNone(v.organise())

        # We can however paste the object in each of the folders
        for folder in [self.source_folder, self.dest_folder]:
            v = zope.component.getMultiAdapter(
                                (folder, self.request), name="sitemenu")
            menu = v.organise()
            self.assertIsNotNone(menu)
            children_titles = [i['title'] for i in menu['children']]
            self.assertTrue('menu_paste' in children_titles)

        # Now when we give the user the Viewer role, they cannot paste anymore
        # so we should not see "Paste" as an available action.
        setRoles(self.portal, TEST_USER_ID, ('Viewer',))
        for folder in [self.source_folder, self.dest_folder]:
            v = zope.component.getMultiAdapter(
                                (folder, self.request), name="sitemenu")
            menu = v.organise()
            self.assertIsNotNone(menu)
            children_titles = [i['title'] for i in menu['children']]
            self.assertFalse('menu_paste' in children_titles)



from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from plonetheme.nuplone.z3cform.interfaces import INuPloneFormLayer


class NuPloneSkin(IDefaultBrowserLayer, INuPloneFormLayer):
    pass



import logging
from zope.annotation.interfaces import IAnnotations
from zope.i18n import translate
from webhelpers.html.builder import literal
from Products.statusmessages import adapter
from Products.statusmessages import STATUSMESSAGEKEY
from htmllaundry import cleaners 
from htmllaundry.utils import sanitize 

logger = logging.getLogger('plonetheme/nuplone/adapter.py')

HTMLMESSAGEKEY = 'literal-messages'

msgcleaner = \
    cleaners.LaundryCleaner(
            page_structure = True,
            safe_attrs_only = True,
            remove_unknown_tags = False,
            allow_tags = [ "blockquote", "a", "em", "strong", "span"],
            add_nofollow = True,
            scripts = True,
            javascript = True,
            comments = True,
            style = False,
            links = False,
            meta = True,
            processing_instructions = True,
            frames = True,
            annoying_tags = False,
            link_target = "_blank")


class StatusMessage(adapter.StatusMessage):
    """ Overrides the standard IStatusMessage adapter to provide literal string
        support (i.e strings with the '__html__' class).

        This allows us to send html as statusmessages, without it being escaped
        by Chameleon.
    """ 
    def addHTML(self, text, type=u'info'):
        """ Add a HTML status message.
        """
        context = self.context
        annotations = IAnnotations(context)
        text = translate(text, context=context)
        old = annotations.get(
                        HTMLMESSAGEKEY,
                        context.cookies.get(HTMLMESSAGEKEY)
                        )
        value = adapter._encodeCookieValue(text, type, old=old)
        context.response.setCookie(HTMLMESSAGEKEY, value, path='/')
        annotations[HTMLMESSAGEKEY] = value

    def show(self):
        """ Removes all status messages (including HTML) and returns them 
            for display.
        """
        context = self.context
        annotations = IAnnotations(context)
        msgs = annotations.get(STATUSMESSAGEKEY,
                                context.cookies.get(STATUSMESSAGEKEY))
        msgs = msgs and adapter._decodeCookieValue(msgs) or []

        html_msgs = annotations.get(HTMLMESSAGEKEY,
                                context.cookies.get(HTMLMESSAGEKEY))
        html_msgs = html_msgs and adapter._decodeCookieValue(html_msgs) or []

        for msg in html_msgs:
            msg.message = literal(sanitize(msg.message, cleaner=msgcleaner, wrap=None))

        value = msgs + html_msgs
        
        # clear the existing cookie entries, except on responses that don't
        # actually render in the browser (really, these shouldn't render
        # anything so we shouldn't get to this message, but some templates
        # are sloppy).
        if self.context.response.getStatus() not in (301, 302, 304):
            context.cookies[STATUSMESSAGEKEY] = None
            context.response.expireCookie(STATUSMESSAGEKEY, path='/')
            annotations[STATUSMESSAGEKEY] = None

            context.cookies[HTMLMESSAGEKEY] = None
            context.response.expireCookie(HTMLMESSAGEKEY, path='/')
            annotations[HTMLMESSAGEKEY] = None
        
        return value
    
    addHTMLStatusMessage = addHTML
    showStatusMessages = show



import logging
import socket
from Acquisition import aq_inner
from zope import schema
from five import grok
from zope.i18n import translate
from zope.component import queryUtility
from plone.directives import form
from plonetheme.nuplone import MessageFactory as _
from Products.MailHost.MailHost import MailHostError
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form.button import buttonAndHandler
from z3c.appconfig.interfaces import IAppConfig
from plonetheme.nuplone.z3cform.form import FieldWidgetFactory
from plonetheme.nuplone.utils import createEmailTo

log = logging.getLogger(__name__)

TextSpan7 = FieldWidgetFactory("z3c.form.browser.text.TextFieldWidget", klass="span-7")
TextLines4Rows = FieldWidgetFactory("z3c.form.browser.textlines.TextLinesFieldWidget", rows=4)


class IContact(form.Schema):
    name = schema.TextLine(
            title=_(u"label_your_name", default="Your name"),
            required=True)
    form.widget(name="plonetheme.nuplone.skin.contact.TextSpan7")

    email = schema.ASCIILine(
            title=_("label_email", default="Email address"),
            required=True)
    form.widget(email="plonetheme.nuplone.skin.contact.TextSpan7")

    subject = schema.TextLine(
            title=_(u"label_subject", default=u"Subject"),
            required=True)
    form.widget(subject="plonetheme.nuplone.skin.contact.TextSpan7")

    message = schema.Text(
            title=_("label_contact_text", default="Your message"),
            required=True)
    form.widget(message="plonetheme.nuplone.skin.contact.TextLines4Rows")



class ContactForm(form.SchemaForm):
    grok.context(ISiteRoot)
    grok.name("contact")
    grok.require("zope2.Public")

    ignoreContext = True
    schema = IContact
    label = _(u"header_contact", default="Contact")
    default_fieldset_label = None

    @buttonAndHandler(_("button_send", default="Send"), name="send")
    def handleSend(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        appconfig = queryUtility(IAppConfig) or {}
        siteconfig = appconfig.get("site", {})

        subject=_(u"contact_mail_subject", default=u"Contact request: ${subject}", mapping=data)
        subject=translate(subject, context=self.request)

        email=createEmailTo(data["name"], data["email"],
                            siteconfig.get("contact.name", self.context.email_from_name),
                            siteconfig.get("contact.email", self.context.email_from_address),
                            subject,
                            data["message"])
        mh=getToolByName(self.context, "MailHost")
        flash=IStatusMessage(self.request).addStatusMessage
        try:
            mh.send(email)
        except MailHostError, e:
            log.error("MailHost error sending contact form for %s: %s", data["email"], e)
            flash(_(u"error_contactmail", u"An error occured while processing your contact request. Please try again later."), "error")
            return
        except socket.error, e:
            log.error("Socket error sending contact form for %s: %s", data["email"], e[1])
            flash(_(u"error_contactmail", u"An error occured while processing your contact request. Please try again later."), "error")
            return

        flash(_(u"Message sent"), "success")
        self.request.response.redirect(aq_inner(self.context).absolute_url())




from plone.dexterity.interfaces import IDexterityContainer
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from five import grok

grok.templatedir("templates")

class Container(grok.View):
    grok.context(IDexterityContainer)
    grok.layer(NuPloneSkin)
    grok.name("nuplone-view")
    grok.template("frontpage")




import logging
from zope.interface import implements
from zope.interface import Interface
from zope.component import adapts
from zope.component import getUtility
from zope.component import queryMultiAdapter
from plone.tiles.interfaces import ITileDataManager
from plone.tiles.interfaces import ITile
from plone.tiles.tile import Tile
from z3c.appconfig.interfaces import IAppConfig

log = logging.getLogger(__name__)

class IAppConfigTile(ITile):
    """A tile with its configured stored in the application config."""


class AppConfigTile(Tile):
    implements(IAppConfigTile)


class AppConfigTileDataManager(object):
    implements(ITileDataManager)
    adapts(IAppConfigTile)

    def __init__(self, tile):
        self.tile=tile

    def get(self):
        appconfig=getUtility(IAppConfig)
        return appconfig.get("tile:%s" % self.tile.id, {})

    def set(self, data):
        raise NotImplementedError


def getTile(context, request, name):
    appconfig=getUtility(IAppConfig)
    config=appconfig.get("tile:%s" % name, {})
    type=config.get("type", name)
    tile=queryMultiAdapter((context, request), Interface, name=type)
    if tile is None:
        log.warn("Detected reference to non-existing tile '%s' for context %r",
                 name, context)
        return None

    if IAppConfigTile.providedBy(tile):
        tile.id=name

    return tile





from plonetheme.nuplone.auth import LoginChallenger
import logging

log = logging.getLogger(__name__)

def setupVarious(context):
    if context.readDataFile('plonetheme.nuplone.txt') is None:
        return

    site=context.getSite()
    pas=site.acl_users
    if not pas.objectIds([LoginChallenger.meta_type]):
        plugin=LoginChallenger("nuplone-challenger", LoginChallenger.meta_type)
        pas._setObject(plugin.getId(), plugin)
        plugin = getattr(pas, plugin.getId())

        infos=[info for info in pas.plugins.listPluginTypeInfo()
               if plugin.testImplements(info["interface"])]
        plugin.manage_activateInterfaces([info["id"] for info in infos])
        for info in infos:
            for i in range(len(pas.plugins.listPluginIds(info["interface"]))):
                pas.plugins.movePluginsUp(info["interface"], [plugin.getId()])




from five import grok
from zope.interface import Interface

from Acquisition import aq_inner
from OFS.interfaces import ICopyContainer

from Products.CMFCore.ActionInformation import ActionInfo
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName

from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from plonetheme.nuplone.utils import getFactoriesInContext

grok.templatedir("templates")


class Sitemenu(grok.View):
    grok.context(Interface)
    grok.name("sitemenu")
    grok.layer(NuPloneSkin)
    grok.template("sitemenu")

    @property
    def settings_url(self):
        return "%s/@@settings" % self.navroot_url

    def update(self):
        self.view_type = self.request.get("view_type", "view")
        self.actions = self.actions()

    def actions(self):
        """Helper method to generate the contents for the actions menu.
        """
        menu = {"title": _("menu_actions", default=u"Actions")}
        children = menu["children"] = []
        submenu = self.factories()
        if submenu:
            children.append(submenu)
        submenu = self.organise()
        if submenu:
            children.append(submenu)

        if children:
            return menu
        else:
            return None

    def factories(self):
        """Helper method to generate the menu items for creating new content.
        If no content can be created None is returned.
        """
        menu = {"title": _("menu_add_new", default=u"Add new")}
        children = menu["children"] = []
        actions = getFactoriesInContext(self.context)
        actions.sort(key=lambda x: x.title)
        for action in actions:
            children.append({"title": action.title,
                             "description": action.description,
                             "url": action.url,
                            })
        if children:
            return menu
        else:
            return None

    def organise(self):
        """Helper method to generate the menu items for organising content
        (copy/paste/etc.).

        If no organising actions are available None is returned.
        """
        context = aq_inner(self.context)
        context_url = context.absolute_url()
        is_root = ISiteRoot.providedBy(context)
        pa = getToolByName(context, 'portal_actions')
        actions = pa.listActions(object=context,
                                 categories=('folder_buttons',),
                                 ignore_categories=None)
        ec = pa._getExprContext(context)
        actions = [ActionInfo(action, ec) for action in actions]

        menu = {"title": _("menu_organise", default=u"Organise")}
        children = menu["children"] = []
        for a in actions:
            if a['visible'] and a['allowed'] \
                    and a['available'] and not is_root:

                if a['id'] == 'copy' and context.cb_isCopyable():
                    children.append({"title": _("menu_copy", default=u"Copy"),
                                    "url": "%s/@@copy" % context_url})

                elif a['id'] == 'cut' and context.cb_isMoveable():
                    children.append({"title": _("menu_cut", default=u"Cut"),
                                    "url": "%s/@@cut" % context_url})

                elif a['id'] == 'paste' and ICopyContainer.providedBy(context):
                    children.append({
                        "title": _("menu_paste", default=u"Paste"),
                        "url": "%s/@@paste" % context_url
                    })
                elif a['id'] == 'delete':
                    children.append({
                        "title": _("menu_delete", default=u"Delete"),
                        "url": "%s/@@delete" % context_url
                    })
        if children:
            return menu



from zope.interface import Interface
from Acquisition import aq_inner
from Acquisition import aq_parent
import zExceptions
from ZODB.POSException import ConflictError
from OFS.interfaces import ICopySource
from OFS.interfaces import ICopyContainer
from OFS.CopySupport import CopyError
from five import grok
from zope.component import getMultiAdapter
from Products.statusmessages.interfaces import IStatusMessage
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.utils import checkPermission

grok.templatedir("templates")


class Copy(grok.View):
    grok.context(ICopySource)
    grok.layer(NuPloneSkin)
    grok.require("zope2.CopyOrMove")
    grok.name("copy")

    def render(self):
        context=aq_inner(self.context)
        container=aq_parent(context)
        if not context.cb_isCopyable():
            raise zExceptions.Unauthorized

        container.manage_copyObjects(context.getId(), self.request)
        flash=IStatusMessage(self.request).addStatusMessage
        flash(_(u"message_copy_sucess", default=u"Copied"), "notice")
        self.request.response.redirect(context.absolute_url())



class Cut(grok.View):
    grok.context(ICopySource)
    grok.layer(NuPloneSkin)
    grok.require("zope2.CopyOrMove")
    grok.name("cut")

    def render(self):
        context=aq_inner(self.context)
        container=aq_parent(context)
        flash=IStatusMessage(self.request).addStatusMessage

        try:
            container.manage_cutObjects(context.getId(), self.request)
            flash(_("message_cut_success", default=u"Cut."), "notice")
        except CopyError:
            flash(_("message_cut_invalid", default=u"It is not possible to move this object."), "error")

        self.request.response.redirect(context.absolute_url())



class Paste(grok.View):
    grok.context(ICopyContainer)
    grok.layer(NuPloneSkin)
    grok.require("zope2.CopyOrMove")
    grok.name("paste")

    def render(self):
        context=aq_inner(self.context)
        flash=IStatusMessage(self.request).addStatusMessage
        if not context.cb_dataValid():
            raise zExceptions.Unauthorized

        try:
            context.manage_pasteObjects(REQUEST=self.request)
            flash(_("message_paste_succes", default=u"Pasted"), "success")
        except ConflictError:
            raise
        except ValueError:
            flash(_("message_paste_valueerror", default=u"You can not paste the copied data here."), "error")
        except zExceptions.Unauthorized:
            flash(_("message_paste_unauthorized", default=u"You are not allowed to paste here."), "error")
        except CopyError, e:
            if "Insufficient Privileges" in e.message:
                raise zExceptions.Unauthorized
            flash(_("message_paste_generic", default=u"No valid data found in the clipboard."), "error")

        self.request.response.redirect(context.absolute_url())



class Delete(grok.View):
    grok.context(Interface)
    grok.name("delete")
    grok.template("delete")

    def verify(self, container, context):
        if not checkPermission(container, "Delete objects"):
            raise zExceptions.Unauthorized

        return True


    def post(self):
        action=self.request.form.get("action", "cancel")
        context=aq_inner(self.context)
        flash=IStatusMessage(self.request).addStatusMessage

        if action=="cancel":
            flash(_("message_delete_cancel", default=u"Deletion cancelled"), "notice")
            self.request.response.redirect(context.absolute_url())
        elif action=="delete":
            authenticator=getMultiAdapter((self.context, self.request), name=u"authenticator")
            if not authenticator.verify():
                raise zExceptions.Unauthorized

            container=aq_parent(context)
            container.manage_delObjects([context.getId()])
            flash(_("message_delete_success", default=u"Object removed"), "success")
            self.request.response.redirect(container.absolute_url())


    def update(self):
        context=aq_inner(self.context)
        container=aq_parent(context)

        if not self.verify(container, context):
            return

        super(Delete, self).update()
        if self.request.method=="POST":
            self.post()




import logging
from zope.interface import Interface
from five import grok
from Acquisition import aq_inner
from AccessControl import getSecurityManager
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import ISiteRoot
from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.utils import isAnonymous
from Products.statusmessages.interfaces import IStatusMessage


log=logging.getLogger(__name__)

grok.templatedir("templates")


class Login(grok.View):
    grok.context(Interface)
    grok.layer(NuPloneSkin)
    grok.name("login")
    grok.template("login")

    def homeUrl(self, user):
        """Return a suitable `home' for a user if came_from is unset or invalid.
        """
        put=getToolByName(self.context, "portal_url")
        return put.getPortalObject().absolute_url()

    def update(self):
        user=getSecurityManager().getUser()

        if "came_from" in self.request:
            self.came_from=self.request.came_from
        else:
            self.came_from=self.request.environ.get("HTTP_REFERER")
        if self.came_from:
            if ":" not in self.came_from:
                # Mostly for mechanize/testbrowser which starts with a bogus 'localhost' as referer.
                self.came_from=None
            else:
                put=getToolByName(self.context, "portal_url")
                if not put.isURLInPortal(self.came_from):
                    self.came_from=None

        self.anonymous=isAnonymous(user)
        self.action="%s/@@login" % aq_inner(self.context).absolute_url()

        if not self.anonymous and "login_attempt" in self.request:
            flash=IStatusMessage(self.request).addStatusMessage
            flash(_(u"message_logged_in", default=u"You have been logged in."), "success")
            mt=getToolByName(self.context, "portal_membership")
            mt.loginUser(self.request)
            next=self.came_from or self.homeUrl(user)
            self.request.response.redirect(next)

        self.failed=self.anonymous and "login_attempt" in self.request



class Logout(grok.View):
    grok.context(ISiteRoot)
    grok.layer(NuPloneSkin)
    grok.name("logout")

    def render(self):
        flash=IStatusMessage(self.request).addStatusMessage
        if not isAnonymous():
            mt=getToolByName(self.context, "portal_membership")
            mt.logoutUser(self.request)
            flash(_(u"message_logged_out", default=u"You have been logged out."), "success")
        else:
            flash(_(u"message_already_logged_out", default=u"You were already logged out."), "notice")
        return self.request.response.redirect(aq_inner(self.context).absolute_url())




from setuptools import setup, find_packages
import os

version = '1.4.0'

setup(name='NuPlone',
      version=version,
      description='A new user interface for Plone',
      long_description=open('README.rst').read() + '\n' +
                       open(os.path.join('docs', 'changes.rst')).read(),
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Web Environment',
          'Framework :: Plone :: 4.0',
          'Framework :: Plone :: 4.1',
          'Framework :: Plone :: 4.2',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Operating System :: OS Independent',
          'Programming Language :: JavaScript',
          'Programming Language :: Python :: 2 :: Only',
        ],
      keywords='',
      author='Cornelis Kolbach and Wichert Akkerman',
      author_email='wichert@wiggy.net',
      url='http://packages.python.org/NuPlone',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plonetheme'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Plone >=4.0',
          'Zope2 >=2.12.5',
          'five.pt >= 2.2.0',
          'five.grok',
          'z3c.pt >=1.1.2',
          'plone.tiles',
          'plone.z3cform',
          'plone.app.z3cform >=0.4.10dev',
          'z3c.form [extra] >=2.3.4dev',
          'plone.i18n',
          'plone.autoform',
          'plone.supermodel',
          'plone.directives.form',
          'plone.formwidget.namedfile >1.0b4',
          'z3c.appconfig',
          'p01.widget.password',
          'Products.statusmessages',
          'WebHelpers',
          'htmllaundry >=2.0',
          'zope.i18n',
          'plone.dexterity',
          'z3c.zrtresource',
          'lxml >=2.3.1',  # For security
      ],
      extras_require={
              'tests': ['plone.testing',
                        'plone.app.testing',
                        ],
      },
      )



from plone.tiles import Tile
from Products.CMFCore.utils import getToolByName
from plonetheme.nuplone.utils import viewType

class LanguageTile(Tile):
    def update(self):
        lt=getToolByName(self.context, "portal_languages")
        languages=[l for l in lt.getAvailableLanguageInformation().values()
                   if l["selected"]]
        languages.sort(key=lambda l: l.get("native", l["name"]))
        self.languages=languages
        self.current_language=lt.getPreferredLanguage()

    def __call__(self):
        view_type=viewType(self.context, self.request)
        if view_type in ["add", "edit"]:
            return None

        self.update()
        return self.index()



from Acquisition import aq_inner
from zope.interface import Interface
from five import grok
from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.utils import getPortal
from plonetheme.nuplone.utils import setLanguage
from Products.statusmessages.interfaces import IStatusMessage


class SwitchLanguage(grok.View):
    grok.context(Interface)
    grok.require('zope2.Public')
    grok.name('switch-language')

    def render(self):
        flash = IStatusMessage(self.request).addStatusMessage
        if setLanguage(self.request, self.context):
            flash(_('message_switch_language',
                default=u'Language updated'), 'success')
        else:
            flash(_('message_switch_language_error',
                default=u'Failed to switch language'), 'error')

        next_url = self.request.get('came_from')
        if not next_url:
            next_url = self.request.environ.get('HTTP_REFERER')
        if not next_url or not \
                next_url.startswith(getPortal(self.context).absolute_url()):
            next_url = aq_inner(self.context).absolute_url()

        self.request.response.redirect(next_url)



from Acquisition import aq_inner
from plone.tiles import Tile
from plonetheme.nuplone.utils import getNavigationRoot
from Products.CMFPlone.utils import typesToList
from Products.CMFCore.utils import getToolByName

class TabsTile(Tile):
    def getRoot(self):
        return getNavigationRoot(self.context)


    def update(self):
        context=aq_inner(self.context)
        contextUrl=context.absolute_url()
        navroot=self.getRoot()
        navrootPath="/".join(navroot.getPhysicalPath())
        portal_properties=getToolByName(self.context, "portal_properties")
        use_view_types=portal_properties.site_properties.typesUseViewActionInListings

        query={}
        query["path"]=dict(query=navrootPath, depth=1)
        query["portal_type"]=typesToList(context)
        query["sort_on"]="getObjPositionInParent"
        query["sort_order"]="asc"
        query["is_default_page"]=False

        catalog=getToolByName(context, "portal_catalog")
        results=[{"id" : brain.id,
                  "title" : brain.Title,
                  "url" : "%s/view" % brain.getURL() if brain.portal_type in use_view_types else brain.getURL(),
                  "class" : None}
                 for brain in catalog.searchResults(query)
                 if not brain.exclude_from_nav]
        current=[(len(result["url"]), result["id"]) for result in results
                 if contextUrl.startswith(result["url"])]
        current.sort()
        if current:
            current=current[0][1]
            for result in results:
                if result["id"]==current:
                    result["class"]="current"
                    break

        self.tabs=results
        self.home_url=navroot.absolute_url()


    def __call__(self):
        self.update()
        return self.index()



from zope.interface import Interface
from zope import schema
from five import grok
from plone.directives import form
from plonetheme.nuplone import MessageFactory as _

grok.templatedir("templates")

class ExternalLinkSchema(form.Schema):
    URL = schema.URI(
            title=_("label_url", default=u"URL"),
            required=True)

    title = schema.TextLine(
            title=_("label_title", default=u"Title"),
            required=False)

    new_window = schema.Bool(
            title=_("label_new_window", default=u"Open link in new window"),
            default=True)


class EditLink(form.SchemaForm):
    grok.context(Interface)
    grok.require("cmf.ModifyPortalContent")
    grok.name("edit-link.html")
    grok.template("editlink")

    ignoreContext = True
    schema = ExternalLinkSchema



from Products.CMFPlone.interfaces import IPloneSiteRoot
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from five import grok

grok.templatedir("templates")

class Frontpage(grok.View):
    grok.context(IPloneSiteRoot)
    grok.layer(NuPloneSkin)
    grok.name("nuplone-view")
    grok.template("frontpage")
    grok.require("zope2.View")




from Acquisition import aq_inner
from AccessControl import getSecurityManager
from zope.interface import Interface
from zope.component import queryUtility
from five import grok
from z3c.appconfig.interfaces import IAppConfig
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from plonetheme.nuplone import utils
from plonetheme.nuplone import MessageFactory as _

class Tools(grok.View):
    """Basic view to expose utilties to templates."""

    grok.context(Interface)
    grok.name("tools")
    grok.layer(NuPloneSkin)

    # Workaround for grok weirdness: it puts a __getitem__ on a view which
    # assumes there is a template variable
    template = None

    def __init__(self, *a):
        super(Tools, self).__init__(*a)
        self.user=getSecurityManager().getUser()
        self.anonymous=self.user is None or self.user.getUserName()=="Anonymous User"
        self.portal=utils.getPortal(self.context)
        self.portal_url=self.portal.absolute_url()
        self.navroot=utils.getNavigationRoot(self.context)
        self.navroot_url=self.navroot.absolute_url()
        self.context_url=aq_inner(self.context).absolute_url()

    def render(self):
        """Little trick to make it easier to access this via from a TALES
        expression."""
        return self

    @utils.reify
    def appConfig(self):
        return queryUtility(IAppConfig) or {}

    def view_type(self):
        return utils.viewType(self.context, self.request)

    @utils.reify
    def site_title(self):
        config=self.appConfig
        title=config.get("site", {}).get("title")
        if title:
            return title
        else:
            return _("default_site_title", default=u"Plone")

    def formatDate(self, date, length="long"):
        return utils.formatDate(self.request, date, length)

    def formatTime(self, time, length=None):
        return utils.formatTime(self.request, time, length)

    def formatDatetime(self, timestamp, length="long"):
        return utils.formatDateTime(self.request, timestamp, length)

    def formatDecimal(self, value, length=None):
        return self.request.locale.numbers.getFormatter("decimal", length).format(value)

    def formatPercentage(self, value, length=None):
        return self.request.locale.numbers.getFormatter("percent", length).format(value)

    def countryName(self, code):
        return self.request.locale.displayNames.territories.get(code.upper())

    def languageName(self, code, default=None):
        code=code.lower()
        names=self.request.locale.displayNames.languages
        return names.get(code, default)

    def checkPermission(self, permission):
        return utils.checkPermission(self.context, permission)




import logging
import socket
from Products.MailHost.MailHost import MailHostError
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_inner
from five import grok
from zope import schema
from zope.i18n import translate
from plone.directives import form
from plonetheme.nuplone import MessageFactory as _
from z3c.form.button import buttonAndHandler
from Products.statusmessages.interfaces import IStatusMessage
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plonetheme.nuplone.utils import createEmailTo
from Products.PasswordResetTool.PasswordResetTool import ExpiredRequestError
from Products.PasswordResetTool.PasswordResetTool import InvalidRequestError


log = logging.getLogger(__name__)


class IRequestPasswordReset(form.Schema):
    login = schema.TextLine(
            title=_("label_login", default=u"Login"),
            required=True)



class IPasswordReset(form.Schema):
    login = schema.TextLine(
            title=_("label_login", default=u"Login"),
            required=True)

    password = schema.Password(
            title = _("label_password", default=u"Password"),
            required = True)



class RequestPasswordForm(form.SchemaForm):
    grok.context(ISiteRoot)
    grok.name("request-password-reset")
    grok.require("zope2.Public")

    email_template = ViewPageTemplateFile("templates/pwreset-email.pt")

    ignoreContext = True
    schema = IRequestPasswordReset
    label = _(u"header_password_reset_request", default="Reset password")
    default_fieldset_label = None
    description = _(u"intro_password_reset_request",
            default=u"For security reasons, we store your password encrypted, "
                    u"and cannot mail it to you. If you would like to reset "
                    u"your password, fill out the form below and we will send "
                    u"you an email at the address you gave when you registered "
                    u"to start the process of resetting your password.")


    @buttonAndHandler(_("button_send", default="Send"), name="send")
    def handleSend(self, action):
        flash=IStatusMessage(self.request).addStatusMessage
        data, errors=self.extractData()
        if errors:
            self.status=self.formErrorsMessage
            return

        pas=getToolByName(self.context, "acl_users")
        mt=getToolByName(self.context, "portal_membership")
        user=pas.getUser(data["login"])
        if user is None:
            log.info("Password reset request for unknown user %s" % data["login"])
            flash(_("error_pwreset_unknown_user", default=u"Unknown username."), "error")
            return

        member=mt.getMemberById(user.getId())
        email_address=member.getProperty("email")
        if not email_address:
            flash(_("error_pwreset_no_email", default=u"No known email address for this user."), "error")
            return

        ppr=getToolByName(self.context, "portal_password_reset")
        reset=ppr.requestReset(user.getId())
        portal_url=aq_inner(self.context).absolute_url()
        reset_url="%s/@@reset-password/%s" % (portal_url, reset["randomstring"])

        data["site"]=self.context.title
        subject=_(u"password_reset_subject", default=u"Password reset for ${site}", mapping=data)
        subject=translate(subject, context=self.request)
        body=self.email_template(reset_url=reset_url, login=data["login"], site=self.context.title)

        email=createEmailTo(self.context.email_from_name,
                            self.context.email_from_address,
                            None, email_address,
                            subject, body)

        mh=getToolByName(self.context, "MailHost")
        try:
            mh.send(email)
        except MailHostError, e:
            log.error("MailHost error sending password reset form to %s: %s", email_address, e)
            flash(_(u"error_contactmail", u"An error occured while processing your contact request. Please try again later."), "error")
            return
        except socket.error, e:
            log.error("Socket error sending password reset form to: %s", email_address, e[1])
            flash(_(u"error_contactmail", u"An error occured while processing your contact request. Please try again later."), "error")
            return

        flash(_("info_pwrest_mail_sent",
            default=u"An email with instructions for resetting your password has been sent."),
            "success")
        self.request.response.redirect(portal_url)



class PasswordReset(form.SchemaForm):
    grok.context(ISiteRoot)
    grok.name("reset-password")
    grok.require("zope2.Public")

    randomstring = None

    ignoreContext = True
    schema = IPasswordReset
    label = _(u"header_password_reset", default=u"Reset password")
    description = _(u"intro_password_reset",
            default=u"Please fill out the form below to set your password.")
    default_fieldset_label = None

    def publishTraverse(self, request, name):
        if self.randomstring is not None:
            raise KeyError(name)

        ppr=getToolByName(self.context, "portal_password_reset")
        try:
            ppr.verifyKey(name)
        except (InvalidRequestError, ExpiredRequestError):
            raise KeyError(name)

        self.randomstring=name
        return self


    @buttonAndHandler(_("button_change", default="Change"), name="change")
    def handleChange(self, action):
        data, errors=self.extractData()
        if errors:
            self.status=self.formErrorsMessage
            return
        
        flash=IStatusMessage(self.request).addStatusMessage
        pas=getToolByName(self.context, "acl_users")
        ppr=getToolByName(self.context, "portal_password_reset")
        user=pas.getUser(data["login"])
        ppr.resetPassword(user.getId(), self.randomstring, data["password"])

        flash(_("password_reset", u"Your password has been reset."), "success")
        portal_url=aq_inner(self.context).absolute_url()
        self.request.response.redirect("%s/@@login" % portal_url)




from zope.schema.vocabulary import getVocabularyRegistry
from zope.schema.vocabulary import VocabularyRegistryError

def getVocabulary(field):
    if field.vocabulary is not None:
        return field.vocabulary

    vr=getVocabularyRegistry()
    try:
        return vr.get(None, field.vocabularyName)
    except VocabularyRegistryError:
        return None



